package com.flyfish.framework.generation;

import com.flyfish.framework.annotations.CodeRule;
import com.flyfish.framework.domain.base.Domain;
import com.flyfish.framework.generation.strategy.TransactionCodeRuleStrategy;
import com.flyfish.framework.generation.strategy.UUIDCodeRuleStrategy;
import com.flyfish.framework.utils.ReflectionUtils;
import org.apache.commons.lang3.StringUtils;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 代码规则工具类
 *
 * @author wangyu
 */
public interface CodeRules {

    // 通用uuid生成
    Class<? extends CodeRuleStrategy> UUID = UUIDCodeRuleStrategy.class;

    // 流水号生成
    Class<? extends CodeRuleStrategy> TRANSACTION = TransactionCodeRuleStrategy.class;

    // 静态策略map
    Map<Class<? extends CodeRuleStrategy>, CodeRuleStrategy> strategies = new ConcurrentHashMap<>();

    /**
     * 获取code
     *
     * @param instance 实体
     * @return 结果
     */
    static String getCode(Domain instance) {
        CodeRule rule = instance.getClass().getAnnotation(CodeRule.class);
        if (null != rule && StringUtils.isBlank(instance.getId())) {
            CodeRuleStrategy strategy = strategies.computeIfAbsent(rule.value(), ReflectionUtils::instantiate);
            return strategy.generate(instance);
        }
        return null;
    }
}
